@@ -88,13 +88,6 @@ static int main_thread_exiting;
 
 static pthread_t pthread_threads_counter;
 
-#ifdef NEED_SEPARATE_REGISTER_STACK
-/* Signal masks for the manager.  These have to be global only when clone2
-   is used since it's currently borken wrt signals in the child.  */
-static sigset_t manager_mask;		/* Manager normal signal mask	*/
-static sigset_t manager_mask_all;	/* All bits set.	*/
-#endif
-
 /* Forward declarations */
 
 static int pthread_handle_create(pthread_t *thread, const pthread_attr_t *attr,
@@ -103,19 +96,20 @@ static int pthread_handle_create(pthread_t *thread, const pthread_attr_t *attr,
 				 int report_events,
 				 td_thr_events_t *event_maskp);
 static void pthread_handle_free(pthread_t th_id);
-static void pthread_handle_exit(pthread_descr issuing_thread, int exitcode);
+static void pthread_handle_exit(pthread_descr issuing_thread, int exitcode)
+     __attribute__ ((noreturn));
 static void pthread_reap_children(void);
 static void pthread_kill_all_threads(int sig, int main_thread_also);
 
 /* The server thread managing requests for thread creation and termination */
 
-int __pthread_manager(void *arg)
+int
+__attribute__ ((noreturn))
+__pthread_manager(void *arg)
 {
   int reqfd = (int) (long int) arg;
   struct pollfd ufd;
-#ifndef NEED_SEPARATE_REGISTER_STACK
   sigset_t manager_mask;
-#endif
   int n;
   struct pthread_request request;
 
@@ -133,9 +127,6 @@ int __pthread_manager(void *arg)
   if (__pthread_threads_debug && __pthread_sig_debug > 0)
     sigdelset(&manager_mask, __pthread_sig_debug);
   sigprocmask(SIG_SETMASK, &manager_mask, NULL);
-#ifdef NEED_SEPARATE_REGISTER_STACK
-  sigfillset(&manager_mask_all);
-#endif
   /* Raise our priority to match that of main thread */
   __pthread_manager_adjust_prio(__pthread_main_thread->p_priority);
   /* Synchronize debugging of the thread manager */
@@ -232,14 +223,23 @@ int __pthread_manager_event(void *arg)
 
 /* Process creation */
 
-static int pthread_start_thread(void *arg)
+static int
+__attribute__ ((noreturn))
+pthread_start_thread(void *arg)
 {
   pthread_descr self = (pthread_descr) arg;
   struct pthread_request request;
   void * outcome;
+#if HP_TIMING_AVAIL
+  hp_timing_t tmpclock;
+#endif
   /* Initialize special thread_self processing, if any.  */
 #ifdef INIT_THREAD_SELF
   INIT_THREAD_SELF(self, self->p_nr);
+#endif
+#if HP_TIMING_AVAIL
+  HP_TIMING_NOW (tmpclock);
+  THREAD_SETMEM (self, p_cpuclock_offset, tmpclock);
 #endif
   /* Make sure our pid field is initialized, just in case we get there
      before our father has initialized it. */
@@ -274,11 +274,12 @@ static int pthread_start_thread(void *arg)
   outcome = self->p_start_args.start_routine(THREAD_GETMEM(self,
 							   p_start_args.arg));
   /* Exit with the given return value */
-  pthread_exit(outcome);
-  return 0;
+  __pthread_do_exit(outcome, CURRENT_STACK_FRAME);
 }
 
-static int pthread_start_thread_event(void *arg)
+static int
+__attribute__ ((noreturn))
+pthread_start_thread_event(void *arg)
 {
   pthread_descr self = (pthread_descr) arg;
 
@@ -294,7 +295,7 @@ static int pthread_start_thread_event(void *arg)
   __pthread_unlock (THREAD_GETMEM(self, p_lock));
 
   /* Continue with the real function.  */
-  return pthread_start_thread (arg);
+  pthread_start_thread (arg);
 }
 
 static int pthread_allocate_stack(const pthread_attr_t *attr,
@@ -524,6 +525,7 @@ static int pthread_handle_create(pthread_t *thread, const pthread_attr_t *attr,
   new_thread->p_guardsize = guardsize;
   new_thread->p_header.data.self = new_thread;
   new_thread->p_nr = sseg;
+  new_thread->p_inheritsched = attr ? attr->__inheritsched : 0;
   /* Initialize the thread handle */
   __pthread_init_lock(&__pthread_handles[sseg].h_lock);
   __pthread_handles[sseg].h_descr = new_thread;
@@ -583,17 +585,12 @@ static int pthread_handle_create(pthread_t *thread, const pthread_attr_t *attr,
 	   And there is some argument for changing the __clone2
 	   interface to pass sp and bsp instead, making it more IA64
 	   specific, but allowing stacks to grow outward from each
-	   other, to get less paging and fewer mmaps.  Clone2
-	   currently can't take signals in the child right after
-	   process creation.  Mask them in the child.  It resets the
-	   mask once it starts up.  */
-	  sigprocmask(SIG_SETMASK, &manager_mask_all, NULL);
+	   other, to get less paging and fewer mmaps.  */
 	  pid = __clone2(pthread_start_thread_event,
   		 (void **)new_thread_bottom,
 			 (char *)new_thread - new_thread_bottom,
 			 CLONE_VM | CLONE_FS | CLONE_FILES | CLONE_SIGHAND |
 			 __pthread_sig_cancel, new_thread);
-	  sigprocmask(SIG_SETMASK, &manager_mask, NULL);
 #else
 	  pid = __clone(pthread_start_thread_event, (void **) new_thread,
 			CLONE_VM | CLONE_FS | CLONE_FILES | CLONE_SIGHAND |
@@ -625,13 +622,11 @@ static int pthread_handle_create(pthread_t *thread, const pthread_attr_t *attr,
   if (pid == 0)
     {
 #ifdef NEED_SEPARATE_REGISTER_STACK
-      sigprocmask(SIG_SETMASK, &manager_mask_all, NULL);
       pid = __clone2(pthread_start_thread,
 		     (void **)new_thread_bottom,
                      (char *)new_thread - new_thread_bottom,
 		     CLONE_VM | CLONE_FS | CLONE_FILES | CLONE_SIGHAND |
 		     __pthread_sig_cancel, new_thread);
-      sigprocmask(SIG_SETMASK, &manager_mask, NULL);
 #else
       pid = __clone(pthread_start_thread, (void **) new_thread,
 		    CLONE_VM | CLONE_FS | CLONE_FILES | CLONE_SIGHAND |
